#include "clustersizeplot.h"
#include "qwt_plot_picker.h"
#include "qwt_picker_machine.h"
#include "qwt_scale_engine.h"


ClusterSizePlot::ClusterSizePlot(QWidget *parent) :
    QwtPlot(parent)
{
	nCurves = XyCollectionPlot::maxPlots;
	setAxisScaleEngine(yLeft, new QwtLog10ScaleEngine());

	setCanvasBackground(QColor(Qt::white));

	for(unsigned int i=0;i< nCurves;i++) {
		QwtPlotCurve *curve = new QwtPlotCurve();
		curve->setRenderHint(QwtPlotItem::RenderAntialiased, true);
		curve->attach(this);
		curve->setPen(QPen(QColor::fromHsv(255*i/nCurves,255,255)));
		curves.push_back(curve);
	}

	selectedTemperatureMarker.setLineStyle(QwtPlotMarker::VLine);
	selectedTemperatureMarker.setXValue(50.00);
	selectedTemperatureMarker.setLinePen(QPen(Qt::red));
	selectedTemperatureMarker.attach(this);

	QwtPlotPicker *picker = new QwtPlotPicker(QwtPlot::xBottom, QwtPlot::yLeft, QwtPicker::VLineRubberBand, QwtPicker::AlwaysOn, canvas());
	picker->setStateMachine(new QwtPickerClickPointMachine());
	connect(picker, SIGNAL(selected(QPointF)), this, SLOT(handleClick(QPointF)));

	QwtPlotPicker *tracker = new QwtPlotPicker(QwtPlot::xBottom, QwtPlot::yLeft, QwtPicker::VLineRubberBand, QwtPicker::AlwaysOn, canvas());
	tracker->setStateMachine(new QwtPickerTrackerMachine());
	QPen trackerPen;
	trackerPen.setColor(Qt::red);
	trackerPen.setStyle(Qt::DotLine);
	tracker->setRubberBandPen(trackerPen);
}

void ClusterSizePlot::setData(XyCollectionPlot data) {
	for(unsigned int i=0;i<nCurves;i++) {
		XYSeries *series = data.getSeries(i);
		curves[i]->setData(series);
	}
	replot();
}

void ClusterSizePlot::handleClick(const QPointF &point) {
	setTemperature((double)point.x());
}

void ClusterSizePlot::setTemperature(double temperature) {
	selectedTemperatureMarker.setXValue(temperature);
	replot();
	if(temperature != currentTemperature) {
		currentTemperature = temperature;
		emit temperatureChanged(temperature);
	}
}
