#include "susceptibilityplotwidget.h"
#include "customwidgetplugin.h"


#include "filechooser.h"
#include "reductionchooser.h"
#include "datasetinformation.h"
#include "clustersizeplot.h"


#include <QtPlugin>


MyCustomWidgets::MyCustomWidgets(QObject *parent)
        : QObject(parent)
{
    widgets.append(new SusceptibilityPlugin(this));
    widgets.append(new FileChooserPlugin(this));
	widgets.append(new ReductionChooserPlugin(this));
	widgets.append(new DatasetInformationPlugin(this));
	widgets.append(new ClusterSizePlotPlugin(this));
}

QList<QDesignerCustomWidgetInterface*> MyCustomWidgets::customWidgets() const
{
    return widgets;
}
Q_EXPORT_PLUGIN2(paraspikes-widgets-plugin, MyCustomWidgets)


SusceptibilityPlugin::SusceptibilityPlugin(QObject *parent)
    : PluginInterface(parent)
{
    _name = "SusceptibilityPlotWidget";
    _includeFile = "susceptibilityplotwidget.h";

    _domXml = "<ui language=\"c++\">\n"
                    " <widget class=\"SusceptibilityPlotWidget\" name=\"susceptibilityPlot\">\n"
                    "  <property name=\"geometry\">\n"
                    "   <rect>\n"
                    "    <x>0</x>\n"
                    "    <y>0</y>\n"
                    "    <width>300</width>\n"
                    "    <height>200</height>\n"
                    "   </rect>\n"
                    "  </property>\n"
                    " </widget>\n"
                    "</ui>\n";
}

QWidget *SusceptibilityPlugin::createWidget(QWidget *parent)
{
    return new SusceptibilityPlotWidget(parent);
}

FileChooserPlugin::FileChooserPlugin(QObject *parent) : PluginInterface(parent) {
    _name = "FileChooser";
    _includeFile = "filechooser.h";

    _domXml = "<ui language=\"c++\">\n"
                    " <widget class=\"FileChooser\" name=\"fileChooser\">\n"
                    "  <property name=\"geometry\">\n"
                    "   <rect>\n"
                    "    <x>0</x>\n"
                    "    <y>0</y>\n"
                    "    <width>300</width>\n"
                    "    <height>200</height>\n"
                    "   </rect>\n"
                    "  </property>\n"
                    " </widget>\n"
                    "</ui>\n";
}

QWidget *FileChooserPlugin::createWidget(QWidget *parent)
{
    FileChooser *f = new FileChooser(parent);
    QStringList b;
    b << "Some format";
    f->addFormats(b);
    return f;
}

ReductionChooserPlugin::ReductionChooserPlugin(QObject *parent) : PluginInterface(parent) {
	_name = "ReductionChooser";
	_includeFile = "reductionchooser.h";

	_domXml = "<ui language=\"c++\">\n"
			  " <widget class=\"ReductionChooser\" name=\"reductionChooser\">\n"
			  "  <property name=\"geometry\">\n"
			  "   <rect>\n"
			  "    <x>0</x>\n"
			  "    <y>0</y>\n"
			  "    <width>300</width>\n"
			  "    <height>200</height>\n"
			  "   </rect>\n"
			  "  </property>\n"
			  " </widget>\n"
			  "</ui>\n";
}

QWidget *ReductionChooserPlugin::createWidget(QWidget *parent)
{
	ReductionChooser *f = new ReductionChooser(parent);
	QStringList schemes;
	schemes << "Some scheme";
	f->addSchemes(schemes);
	return f;
}

DatasetInformationPlugin::DatasetInformationPlugin(QObject *parent) : PluginInterface(parent) {
	_name = "DatasetInformation";
	_includeFile = "datasetinformation.h";

	_domXml = "<ui language=\"c++\">\n"
			  " <widget class=\"DatasetInformation\" name=\"datasetInformation\">\n"
			  "  <property name=\"geometry\">\n"
			  "   <rect>\n"
			  "    <x>0</x>\n"
			  "    <y>0</y>\n"
			  "    <width>300</width>\n"
			  "    <height>200</height>\n"
			  "   </rect>\n"
			  "  </property>\n"
			  " </widget>\n"
			  "</ui>\n";
}

QWidget *DatasetInformationPlugin::createWidget(QWidget *parent) {
	DatasetInformation *d = new DatasetInformation(parent);
	return d;
}

ClusterSizePlotPlugin::ClusterSizePlotPlugin(QObject *parent) : PluginInterface(parent) {
	_name = "ClusterSizePlot";
	_includeFile = "clustersizeplot.h";

	_domXml = "<ui language=\"c++\">\n"
			  " <widget class=\"ClusterSizePlot\" name=\"clusterSizePlot\">\n"
			  "  <property name=\"geometry\">\n"
			  "   <rect>\n"
			  "    <x>0</x>\n"
			  "    <y>0</y>\n"
			  "    <width>300</width>\n"
			  "    <height>200</height>\n"
			  "   </rect>\n"
			  "  </property>\n"
			  " </widget>\n"
			  "</ui>\n";
}

QWidget *ClusterSizePlotPlugin::createWidget(QWidget *parent) {
	ClusterSizePlot * c = new ClusterSizePlot(parent);
	return c;
}
