#include <QtGui>
#include "susceptibilityplotwidget.h"
#include "qwt_legend.h"
#include "qwt_plot_picker.h"
#include "qwt_picker_machine.h"

#include <stdio.h>
#include <iostream>


SusceptibilityPlotWidget::SusceptibilityPlotWidget(QWidget *parent) : QwtPlot(parent) {
    setCanvasBackground(QColor(Qt::white));
	//setAxisTitle(QwtPlot::yLeft, "Susceptibility * temperature");
	//setAxisTitle(QwtPlot::xBottom, "Temperature");

	//QwtLegend *legend = new QwtLegend;
	//legend->setFrameStyle(QFrame::Box | QFrame::Sunken);
	//insertLegend(legend, BottomLegend);

	//curve.setTitle("Susceptibility");
    curve.setRenderHint(QwtPlotItem::RenderAntialiased);
    curve.setPen(QPen(Qt::blue));
    curve.attach(this);



    selectedTemperatureMarker.setLineStyle(QwtPlotMarker::VLine);
    selectedTemperatureMarker.setXValue(50.00);
    selectedTemperatureMarker.setLinePen(QPen(Qt::red));
    selectedTemperatureMarker.attach(this);


    QwtPlotPicker *picker = new QwtPlotPicker(QwtPlot::xBottom, QwtPlot::yLeft, QwtPicker::VLineRubberBand, QwtPicker::AlwaysOn, canvas());
    picker->setStateMachine(new QwtPickerClickPointMachine());
    connect(picker, SIGNAL(selected(QPointF)), this, SLOT(handleClick(QPointF)));

    QwtPlotPicker *tracker = new QwtPlotPicker(QwtPlot::xBottom, QwtPlot::yLeft, QwtPicker::VLineRubberBand, QwtPicker::AlwaysOn, canvas());
    tracker->setStateMachine(new QwtPickerTrackerMachine());
    QPen trackerPen;
    trackerPen.setColor(Qt::red);
    trackerPen.setStyle(Qt::DotLine);
    tracker->setRubberBandPen(trackerPen);


}

void SusceptibilityPlotWidget::setSuPlot(XyPlot &plot) {
	XYSeries *s = plot.to_X_XY_Series();
	curve.setData(s);
	replot();
}

void SusceptibilityPlotWidget::handleClick(const QPointF &point) {
    setTemperature((double)point.x());
}

void SusceptibilityPlotWidget::setTemperature(double temperature) {
    selectedTemperatureMarker.setXValue(temperature);
    replot();
	if(temperature != currentTemperature) {
		currentTemperature = temperature;
		emit temperatureChanged(temperature);
	}
}
