#include "clusterquality.h"

#include <gsl/gsl_matrix.h>
#include <gsl/gsl_statistics.h>
#include <gsl/gsl_linalg.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_cdf.h>
#include <assert.h>
#include <string.h>
#include <algorithm>
#include <set>
#include <iostream>
#include <fstream>
#include "macros.h"


static void
printMatrixTheHardWay(gsl_matrix*m)
{
	/* print matrix the hard way */
	printf("Matrix m\n");
	for (unsigned int row=0;row<m->size1;row++)
	{
		for (unsigned int col=0;col<m->size2;col++)
		{
			printf("%f ",gsl_matrix_get(m,row,col));
		}
		printf("\n");
	}
	printf("\n");
}

ClusterQuality::ClusterQuality(std::vector<std::vector<double> > &points, std::vector<std::vector<int> > &clusters)
{

	mahalForCluster.resize(clusters.size());
	mahalForNoise.resize(clusters.size());
	this->points = &points;
	this->clusters = &clusters;

}




void
ClusterQuality::calculateLratioQualityAndIsolationDistance() {

	int numSpikes = points->size();
	int numFeatures = (*points)[0].size();
	int numClusters = clusters->size();

	double *clusterMean = new double[numFeatures];


	gsl_vector *clusterMeanVector = gsl_vector_alloc(numFeatures);

	gsl_matrix *covarianceMatrix = gsl_matrix_alloc(numFeatures, numFeatures);

	gsl_matrix *spikeMat = gsl_matrix_alloc(numFeatures, numSpikes);
	for(int spike = 0; spike < numSpikes; ++spike) {
		for(int feature = 0; feature < numFeatures; ++feature) {
			gsl_matrix_set(spikeMat, feature, spike, (*points)[ spike ] [feature]);
		}
	}

	setlocale(LC_NUMERIC, "C");
	FILE *isolationFile = fopen("isolation.txt", "w");
	fprintf(isolationFile, "noiseData = [[],[],[],[],[],[],[]]\nclusterData=[[],[],[],[],[],[],[]]\n");

	//WHEREAMI();
	//printf("SpikeMat\n");
	//printMatrixTheHardWay(spikeMat);


	for(int currentCluster = 0; currentCluster < numClusters; ++currentCluster) {
		std::cout << "Cluster " << currentCluster << ":" << std::endl;

		int clusterSize = (*clusters)[currentCluster].size();

		if(clusterSize < numFeatures) {
			std::cout << "Size " << clusterSize << " is fewer than numFeatures (" << numFeatures << ") spikes. Not considered" << std::endl;
			continue;
		}

		gsl_matrix *mForCluster = gsl_matrix_alloc(numFeatures, clusterSize);
		for(int spikeInCluster = 0; spikeInCluster < clusterSize; ++spikeInCluster) {
			for(int feature = 0; feature < numFeatures; ++feature) {
				int spikeGlobalIndex = (*clusters)[currentCluster][spikeInCluster];
				gsl_matrix_set(mForCluster, feature, spikeInCluster, (*points)[ spikeGlobalIndex ] [feature]);
			}
		}

		memset(clusterMean, 0, sizeof(double)*numFeatures);

		for(int i = 0; i < clusterSize; ++i) {
			for(int j = 0; j < numFeatures; ++j) {
				clusterMean[j] += (*points)[ (*clusters)[currentCluster][i] ][j];
			}
		}
		for(int j = 0; j < numFeatures; ++j) {
			clusterMean[j] /= clusterSize;
		}

		gsl_matrix *mForClusterCentered = gsl_matrix_alloc(numFeatures, clusterSize);
		for(int spikeInCluster = 0; spikeInCluster < clusterSize; ++spikeInCluster) {
			for(int feature = 0; feature < numFeatures; ++feature) {
				gsl_matrix_set(mForClusterCentered, feature, spikeInCluster, gsl_matrix_get(mForCluster, feature, spikeInCluster) - clusterMean[feature]);
			}
		}

		//WHEREAMI();
		//printf("mforclustercentered");
		//printMatrixTheHardWay(mForClusterCentered);


		// std::cout << "mean: " << mean[0] << ", " << mean[1] << std::endl;

		//printf("mForClusterCentered:\n"); printMatrixTheHardWay(mForClusterCentered);

		for (int dim1 = 0; dim1 < numFeatures; dim1++) {
			for (int dim2 = 0; dim2 < numFeatures; dim2++) {
				gsl_vector_view a = gsl_matrix_row(mForCluster, dim1);
				gsl_vector_view b = gsl_matrix_row(mForCluster, dim2);
				double v = gsl_stats_covariance(a.vector.data, a.vector.stride, b.vector.data, b.vector.stride, a.vector.size);
				gsl_matrix_set(covarianceMatrix, dim1, dim2, v);
			}
		}
		//printMatrixTheHardWay(covarianceMatrix);

		// Matmul - not accurate enough?
		// gsl_blas_dgemm(CblasNoTrans, CblasTrans, 1, mForClusterCentered, mForClusterCentered, 0, covarianceMatrix); // TODO unsure if it should be centered or not...
		// gsl_matrix_scale(covarianceMatrix, 1.0 / clusterSize);


		//printf("Covar matrix\n"); printMatrixTheHardWay(covarianceMatrix);
		//gsl_matrix_fprintf(stdout,covarianceMatrix,"%f");


		//gsl_linalg_cholesky_decomp(covarianceMatrix);
		//gsl_linalg_cholesky_invert(covarianceMatrix);

		gsl_matrix *covarianceInverseMatrix = gsl_matrix_calloc(covarianceMatrix->size1, covarianceMatrix->size2);
		gsl_permutation *p = gsl_permutation_alloc(numFeatures);
		int signum = 0;

//		WHEREAMI();
		//printMatrixTheHardWay(covarianceMatrix);

		gsl_linalg_LU_decomp (covarianceMatrix, p, &signum);

		// Check for singularity:
		double determinant = gsl_linalg_LU_det(covarianceMatrix, signum);
		if(determinant== 0) {
			std::cout << "Covariance matrix for cluster " << currentCluster << " is singular." << std::endl;
			continue;
		}

		gsl_linalg_LU_invert (covarianceMatrix, p, covarianceInverseMatrix);
		gsl_permutation_free(p);

		//printf("Covar matrix INVERSED!\n");
		//printMatrixTheHardWay(covarianceMatrix);

		for(int feature = 0; feature < numFeatures; ++feature) {
			gsl_vector_set(clusterMeanVector, feature, clusterMean[feature]);
		}

		// Calculate mahalanobis and L for clusters
		double lc = 0;
		for(int spike=0; spike < numSpikes; spike++) {
			//printf("Spike %i................\n", spike);

			gsl_vector* centeredSpikeVector = gsl_vector_alloc_col_from_matrix(spikeMat, spike);
			gsl_vector_sub(centeredSpikeVector, clusterMeanVector);

			// x_i^T * sigma^-1 * x_i
			gsl_vector* tempVector = gsl_vector_alloc(numFeatures);
			gsl_blas_dgemv(CblasNoTrans, 1, covarianceInverseMatrix, centeredSpikeVector, 0, tempVector);
			double d2;
			gsl_blas_ddot(centeredSpikeVector, tempVector, &d2);
			gsl_vector_free(tempVector);

			//std::cout << "d2 for " << spike << ": " << d2;
			//TODO this check may be optimized by a more convenient data structure
			if(std::find((*clusters)[currentCluster].begin(), (*clusters)[currentCluster].end(), spike) == (*clusters)[currentCluster].end()) { // Not in cluster
				//double p = gsl_cdf_chisq_P(d2, numFeatures);
				double q = gsl_cdf_chisq_Q(d2, numFeatures); // = 1-p
				lc += q;
				mahalForNoise[currentCluster].push_back(d2);
				//std::cout << " (noise)" << std::endl;
			} else {
				mahalForCluster[currentCluster].push_back(d2);
				//std::cout << " (cluster)" << std::endl;
			}
			gsl_vector_free(centeredSpikeVector);

		}

		// Order mahalanobis
		std::sort(mahalForNoise[currentCluster].begin(), mahalForNoise[currentCluster].end());
		std::sort(mahalForCluster[currentCluster].begin(), mahalForCluster[currentCluster].end());

		if(mahalForNoise[currentCluster].size() < mahalForCluster[currentCluster].size()) {
			printf("Isolation distance undefined (this cluster contains more than half of the data)\n");
		} else {
			printf("Isolation distance = %f\n", mahalForNoise[currentCluster][clusterSize]);
		}

		// Mahal noise
		fprintf(isolationFile, "noiseData[%d] = [", currentCluster);
		for(unsigned int i=0;i< mahalForNoise[currentCluster].size(); i++) {
			fprintf(isolationFile, "%f" ,mahalForNoise[currentCluster][i]);
			if(i != mahalForNoise[currentCluster].size() -1)
				fprintf(isolationFile,", ");
		}
		fprintf(isolationFile, "]\n");

		// Mahal cluster
		fprintf(isolationFile, "clusterData[%d] = [", currentCluster);
		for(unsigned int i=0;i< mahalForCluster[currentCluster].size(); i++) {
			fprintf(isolationFile, "%f" ,mahalForCluster[currentCluster][i]);
			if(i != mahalForCluster[currentCluster].size() -1)
				fprintf(isolationFile,", ");
		}
		fprintf(isolationFile, "]\n");

		gsl_matrix_free(covarianceInverseMatrix);
		printf("L(c) = %f\n", lc);
		printf("Lratio = %f\n\n", lc/clusterSize);

		fflush(stdout);

		gsl_matrix_free(mForCluster);
	}
	fclose(isolationFile);
	delete[] clusterMean;
	gsl_vector_free(clusterMeanVector);
	gsl_matrix_free(covarianceMatrix);



}

double ClusterQuality::calculateSilhouette(unsigned int cluster) {
	int numDimensions = (*points)[0].size();	
	unsigned int clusterSize = (*clusters)[cluster].size();

	std::vector<double> a(clusterSize, 0);
	std::vector<double> b(clusterSize, 0);
	std::vector<double> c(clusterSize, 0);
	std::vector<double> tmp(clusterSize, 0);

	/*std::cout << "Cluster info: " << std::endl;
	for (unsigned int i = 0; i < clusters->size(); ++i) {
		std::cout << "#" << i << ": " << (*clusters)[i].size() << std::endl;
	}*/

	// check the average internal cluster distance (cohesion)
	for(unsigned int pointInCluster = 0; pointInCluster < clusterSize; ++pointInCluster) {
		for(unsigned int otherPointInCluster = 0; otherPointInCluster < clusterSize; ++otherPointInCluster) {
			if(pointInCluster == otherPointInCluster)
				continue;

			int indexi = (*clusters)[cluster][pointInCluster];
			int indexj = (*clusters)[cluster][otherPointInCluster];

			double t = 0.0;
			for(int dimension = 0; dimension < numDimensions; ++dimension)
				t += ((*points)[indexi][dimension] - (*points)[indexj][dimension]) *
									((*points)[indexi][dimension] - (*points)[indexj][dimension]);
			a[pointInCluster] += sqrt(t);
		}		
		a[pointInCluster] /= clusterSize; //-1?
	}

	// check the distances to the other clusters (separation)
	int first = true;
	for(unsigned int neighborCluster = 0; neighborCluster < clusters->size(); ++neighborCluster) {
		if(neighborCluster == cluster)
			continue;		

		tmp.clear();
		tmp.resize(clusterSize, 0);
		for(unsigned int pointInCluster = 0; pointInCluster < clusterSize; ++pointInCluster) {
			int pointInClusterIndex = (*clusters)[cluster][pointInCluster];
			for(unsigned int pointInNeighborCluster = 0; pointInNeighborCluster < (*clusters)[neighborCluster].size(); ++pointInNeighborCluster) {
				int pointInNeighborClusterIndex = (*clusters)[neighborCluster][pointInNeighborCluster];

				assert(pointInClusterIndex != pointInNeighborClusterIndex); // assumes all classified

				double t = 0.0;
				for(int dimension = 0; dimension < numDimensions; ++dimension)
					t += ((*points)[pointInClusterIndex][dimension] - (*points)[pointInNeighborClusterIndex][dimension]) *
										((*points)[pointInClusterIndex][dimension] - (*points)[pointInNeighborClusterIndex][dimension]);
				tmp[pointInCluster] += sqrt(t);
			}
			tmp[pointInCluster] /= (*clusters)[neighborCluster].size();
			if(tmp[pointInCluster] < b[pointInCluster] || first)
				b[pointInCluster] = tmp[pointInCluster];
		}
		//std::cout << "Size: " << (*clusters)[neighborCluster].size() << std::endl;
		//TODO: can optimize this (early return if tmp > b)

		first = false;
	}

	for(unsigned int i = 0; i < a.size(); ++i) {
		if(a[i] < b[i])
			c[i] = 1.0 - a[i] / b[i];
		else if(a[i] > b[i])
			c[i] = b[i] / a[i] - 1.0;
	}
	std::sort(c.begin(), c.end());

	/*std::ofstream fout;
	char filename[256] = {0};
	sprintf(filename, "silhouette%i.txt", cluster);
	fout.open(filename);
	fout << "Points in cluster " << cluster << ": " << a.size() << std::endl;
	fout << "Point in other clusters: " << b.size() << std::endl;

	fout << std::endl;
	for(unsigned int i = 0; i < a.size(); ++i)
		fout << a[i] << ", ";
	fout << std::endl;
	for(unsigned int i = 0; i < b.size(); ++i)
		fout << b[i] << ", ";
	fout << std::endl;
	for(unsigned int i = 0; i < c.size(); ++i)
		fout << c[i] << ", ";
	fout << std::endl;

	fout.close();*/

	double averageSilhouetteQuality = 0.0;
	for(std::vector<double>::const_iterator it = c.begin(); it != c.end(); ++it)
		averageSilhouetteQuality += *it;
	averageSilhouetteQuality /= clusterSize;

	return averageSilhouetteQuality;
}

double ClusterQuality::calculateAverageSilhouettes() {
	std::vector<double> averageSilhouettes(clusters->size());

	#pragma omp parallel
	for(unsigned int i = 0; i < averageSilhouettes.size(); ++i)
		averageSilhouettes[i] = calculateSilhouette(i);

	std::cout << "Average silhouettes:" << std::endl;
	for(unsigned int i = 0; i < averageSilhouettes.size(); ++i)
		std::cout << i << ": " << averageSilhouettes[i] << std::endl;

	//std::cout << "Global silhouette:";
	double globalSilhouette = 0.0;
	for(unsigned int i = 0; i < averageSilhouettes.size(); ++i) {
		globalSilhouette += averageSilhouettes[i] * (*clusters)[i].size();
	}
	globalSilhouette /= points->size();
	//std::cout << globalSilhouette << "." << std::endl;

	//std::cout << "Total silhouette:";
	double totalSilhouette = 0.0;
	for(unsigned int i = 0; i < averageSilhouettes.size(); ++i)
		totalSilhouette += averageSilhouettes[i];
	//std::cout << "Total silhouette:" << totalSilhouette << std::endl;

	//std::cout << "Average total silhouette:" << totalSilhouette / averageSilhouettes.size() << std::endl;

	std::cout << "Global/total/average silhouette:" << globalSilhouette << "," << totalSilhouette << "," << totalSilhouette / averageSilhouettes.size() << std::endl;

	//return averageSilhouettes;
	return globalSilhouette;
}

double ClusterQuality::fmeasure(std::vector<int> cluster, std::vector<int> cutCluster, int flag) { //TODO unclean code, could be split in 3 methods
	sort(cluster.begin(), cluster.end());
	sort(cutCluster.begin(), cutCluster.end());

	std::vector<int> res(cluster.size());
	std::vector<int>::iterator resEnd = std::set_intersection(cluster.begin(), cluster.end(), cutCluster.begin(), cutCluster.end(), res.begin());

	if(resEnd == res.begin())
		return 0;

	double intersectionSize = int(resEnd - res.begin());
	double precision = intersectionSize / cluster.size();
	double recall = intersectionSize / cutCluster.size();

	if(flag == 1) {
		return precision;
	} else if(flag == 2) {
		return recall;
	}

	return 2.0 * precision * recall / (precision + recall);
}


void ClusterQuality::printFmeasurePrecisionAndRecall(std::vector<std::vector<int> > cutClusters) {

	std::cout << "Precision. Cluster index downwards, Cut cluster Index -->" << std::endl;
	for(unsigned int cutCluster=0; cutCluster < cutClusters.size(); cutCluster++) {
			for(unsigned int cluster=0; cluster < clusters->size(); cluster++) {
			std::cout << fmeasure((*clusters)[cluster], cutClusters[cutCluster], 1) << "\t";
		}
		std::cout << std::endl;
	}
	std::cout << std::endl;

	std::cout << "Recall" << std::endl;
	for(unsigned int cutCluster=0; cutCluster < cutClusters.size(); cutCluster++) {
		for(unsigned int cluster=0; cluster < clusters->size(); cluster++) {
			std::cout << fmeasure((*clusters)[cluster], cutClusters[cutCluster], 2) << "\t";
		}
		std::cout << std::endl;
	}
	std::cout << std::endl;

	std::cout << "F-measure" << std::endl;
	for(unsigned int cutCluster=0; cutCluster < cutClusters.size(); cutCluster++) {
		for(unsigned int cluster=0; cluster < clusters->size(); cluster++) {
			std::cout << fmeasure((*clusters)[cluster], cutClusters[cutCluster], 0) << "\t";
		}
		std::cout << std::endl;
	}
	std::cout << std::endl;

}
