#include "dataexporter.h"

#include <fstream>

void DataExporter::saveSpikeData(const char *filename, std::vector<std::vector<double> > data, unsigned int numSamples, unsigned int numDimensions)
{
	std::fstream f;
	f.open(filename, std::ios::out);

	if (!f.is_open()) {
		// Mangling Qt into this isn't ideal
		//QMessageBox::warning(NULL, "Unknown error while loading the file", "Unknown type");
		return;
	}

	f << numSamples << std::endl;
	f << numDimensions << std::endl;

	for(unsigned int sample=0; sample < numSamples; ++sample) {
		for(unsigned int dimension = 0; dimension < numDimensions; ++dimension) {
			f << data[sample][dimension];
			if(dimension+1 != numDimensions)
				 f << ",";
		}
		if(sample+1 != numSamples)
			f << std::endl;
	}
}

void DataExporter::saveCutFile(std::vector <int> &clusterIds, const char *filename)
{
	std::fstream f;
	f.open(filename, std::ios::out);

	int numClusters = 0;
	for(std::vector <int>::iterator it = clusterIds.begin(); it != clusterIds.end(); ++it) {
		if(*it > numClusters)
			numClusters = *it;
	}
	numClusters += 1; // since 0 is also a cluster

	f << "n_clusters: " << numClusters << std::endl;
	f << "n_channels: 4" << std::endl;
	f << "n_params: 2" << std::endl;
	f << "times_used_in_Vt:    0    0    0    0" << std::endl;

	for(int i = 0; i < numClusters; ++i) {
		 f << "cluster: " << i << " center:    0    0    0    0    0    0    0    0" << std::endl;
		 f << "               min:    0    0    0    0    0    0    0    0" << std::endl;
		 f << "               max:    0    0    0    0    0    0    0    0" << std::endl;
	}
	f << std::endl;



	f << "Exact_cut_for: TODO spikes: " << clusterIds.size();

	for(unsigned int i = 0; i < clusterIds.size(); ++i) {
		if(!(i % 25))
			f << std::endl;
		  f << "  " << clusterIds[i];
	}
}


void DataExporter::saveWithoutClusterZero(InputDataset *in, std::vector<std::vector<int> > &cutFileClusters, const char *filename)
{
	std::vector<std::vector<double> > newPoints;
	std::vector<int> newClusterIndexes;

	for(unsigned int clusterIndex=1; clusterIndex < cutFileClusters.size(); clusterIndex++) {
		for(std::vector<int>::iterator it = cutFileClusters[clusterIndex].begin(); it != cutFileClusters[clusterIndex].end(); it++) {
			newPoints.push_back(in->data[*it]);
			newClusterIndexes.push_back(clusterIndex - 1); // because removing 0
		}
	}
	unsigned numSamples = newPoints.size();

	saveSpikeData(filename, newPoints, numSamples, in->nDimensions);

	char cutFilename[200] = {0}; sprintf(cutFilename, "%s.cut", filename);

	saveCutFile(newClusterIndexes, cutFilename);
}
