#include "dataloader.h"
#include "dataexporter.h"
#include <QTextStream>
#include <fstream>
#include <iostream>
#include <istream>
#include <sstream>
#include <iomanip>
#include <float.h>

#include <boost/algorithm/string/trim.hpp>
#include <boost/algorithm/string/split.hpp>
#include <boost/tokenizer.hpp>

bool unique_path(int *path, int v, int level)
{
	for(int i = 0; i < level; ++i)
		if(path[i] == v)
			return false;

	return true;
}

void compute_shortest_path_rec(float *t, int *path, float length, int *shortestPath, float *shortestLength, int level, int manualCutMaxLevel, int autoCutMaxLevel)
{
	float localLength;

	for(int a = 0; a < autoCutMaxLevel; ++a)  {
		if(!unique_path(path, a, level))
			continue;

		int *localPath = new int[manualCutMaxLevel];
		memcpy(localPath, path, sizeof(int) * manualCutMaxLevel);

		localPath[level] = a;
		localLength = length + t[localPath[level] + autoCutMaxLevel*level];

		if(localLength > *shortestLength)
			continue;

		//if(level == (manualCutMaxLevel<autoCutMaxLevel?manualCutMaxLevel:autoCutMaxLevel)-1) {
		if(level == (manualCutMaxLevel-1)) {
			if(localLength < *shortestLength) {
				*shortestLength = localLength;
				for(int i = 0; i < manualCutMaxLevel; ++i)
					shortestPath[i] = localPath[i] + autoCutMaxLevel*i;
			}
		} else {
			compute_shortest_path_rec(t, localPath, localLength, shortestPath, shortestLength, level+1, manualCutMaxLevel, autoCutMaxLevel);
		}

		delete localPath;
	}
}

// prints manual X auto (row X col)
float similaritySumOfDifferences(double *manualCutData, unsigned int numManualCutData, double *autoCutData, unsigned int numAutoCutData, int numDimensions, std::vector<int> &out)
{
	int numAutoCutDataZeroPadded = numAutoCutData;
	if(numAutoCutData < numManualCutData)
		numAutoCutDataZeroPadded = numManualCutData;

	float *similarityMatrix = new float[numManualCutData * numAutoCutDataZeroPadded];
	memset(similarityMatrix, 0, sizeof(float) * numManualCutData * numAutoCutDataZeroPadded);

	std::cout << "Sum of absolute differences (cutter x kmeans in " << numManualCutData << " x " << numAutoCutData << "):" << std::endl;
	std::cout << "\t";
	for(int i = 0; i < numAutoCutDataZeroPadded; ++i)
		std::cout << "   " << std::setw(10) << std::setiosflags(std::ios::fixed) << std::setprecision(2) << i;
	std::cout << std::endl;

	for(unsigned int a = 0; a < numManualCutData; ++a) {
		std::cout << "\t" << a << ": ";
		for(unsigned int b = 0; b < numAutoCutData; ++b) {
			float diff = 0;
			for(int dimension = 0; dimension < numDimensions; ++dimension) {
				diff += fabs(manualCutData[a * numDimensions + dimension] - autoCutData[b * numDimensions + dimension]);
			}
			//diff /= (float)(numData*numDimensions);
			//cout << "\t" << setprecision(2) << setw(6) << fabs(diff);
			std::cout << "   " << std::setw(10) << std::setiosflags(std::ios::fixed) << std::setprecision(2) << (diff);
			similarityMatrix[a * numAutoCutDataZeroPadded + b] = diff;
		}
		std::cout << std::endl;
	}

	int pathLength = numManualCutData;
	/*if(numManualCutData > numAutoCutData)
		pathLength = numAutoCutData;
	else
		pathLength = numManualCutData;*/

	std::cout << "pathlength: " << pathLength << std::endl;

	int *path = new int[pathLength];
	memset(path, -1, sizeof(int) * pathLength);
	float length = FLT_MAX;

	int *localPath = new int[pathLength];
	memset(localPath, -1, sizeof(int) * pathLength);
	float localLength = 0;
	compute_shortest_path_rec(similarityMatrix, localPath, localLength, path, &length, 0, pathLength, numAutoCutDataZeroPadded);

	std::cout << "Shortest path:\n";
	float total_length = 0;

	out.clear();
	out.resize(pathLength);
	for(int i = 0; i < pathLength; ++i) {
		out[i] = path[i] % numAutoCutData;
		std::cout << "(" << i << ", " << path[i] % numAutoCutDataZeroPadded << "): " << similarityMatrix[path[i]] << std::endl;
		total_length += similarityMatrix[path[i]];
	}

	std::cout << "Total length: " << total_length << std::endl;


	/*
	if(strlen(filename) > 0) {
		std::ofstream f;
		f.open(filename);

		for(unsigned int spike = 0; spike < numData1; ++spike) {
			for(int point = 0; point < numDimensions; ++point)
				f << data1[spike * numDimensions + point] << "\t";
			f << endl;
			for(int point = 0; point < numDimensions; ++point)
				f << data2[(path[spike] % numData2) * numDimensions + point] << "\t";
			f << endl << endl;
		}
	}*/

	delete[] similarityMatrix;
	delete[] localPath;
	delete[] path;

	return total_length;
}



InputDataset* DataLoader::loadFile(char *filename, int filetypeIndex) {
	return loadFile(QString(filename), filetypeIndex);
}

InputDataset* DataLoader::loadFile(QString filename, int filetypeIndex) {
    Filetype filetype = Filetype(filetypeIndex);
    if(filetype == ASCII) {
        return loadAsciiFile(filename);
    } else if(filetype == AXONA) {
		return loadAxonaFile(filename);
    } else {
		//QMessageBox::warning(NULL, "Filetype not supported yet", "Unknown type");
        return NULL;
    }
}

InputDataset* DataLoader::loadAsciiFile(QString filename) {
    QFile inFile(filename);
	bool fileOpened = inFile.open(QIODevice::ReadOnly);

	if(!fileOpened) {
		//QMessageBox::warning(NULL, "Unknown error while loading the file", "Unknown type");
		return NULL;
	}

    QTextStream inStream(&inFile);

    int nSamples = inStream.readLine().toInt();
    int nDimensions = inStream.readLine().toInt();

    std::vector<std::vector<double> > data(nSamples);

    for(int i=0;i<nSamples;i++) {
        QString line = inStream.readLine();
		QStringList splitLine = line.split(",");
		std::vector<double> sample(nDimensions); //TODO use the existing vector instead
		for(int j=0;j<nDimensions;j++) {
			sample[j] = splitLine.at(j).toDouble();
        }
        data[i] = sample;
    }
	//TODO: # channels should be specified in the ASCII file.
	InputDataset* dataset = new InputDataset(nDimensions, nSamples, 1);
    dataset->data = data;
    return dataset;
}

InputDataset* DataLoader::loadAxonaFile(QString filename) {
	std::fstream f;

	f.open(filename.toStdString().c_str(), std::ios::in);

	if (!f.is_open()) {
		// Mangling Qt into this isn't ideal
		//QMessageBox::warning(NULL, "Unknown error while loading the file", "Unknown type");
		return NULL;
	}

	std::string line;
	std::map<std::string, std::string> metaData;

	void *data = NULL;

	long binaryDataStart = 0;
	while (getline(f, line)) {

		if(line.find("data_start") != std::string::npos)
		{

			f.seekg(0, std::ios::end);

			//FIXME: remember end of data marker
			long binaryDataEnd = f.tellg();

			//remember to add whatever is in line as the marker since the binary data starts on the same LINE
			long startExtra = line.substr(0, std::string("data_start").size()).size();
			binaryDataStart += startExtra;

			// 12 is the size of the redundant end marker
			long binarySize = binaryDataEnd - binaryDataStart - 12;

			std::cout << "Binary data starting at " << binaryDataStart << ", with a size of: " << binarySize << "." << std::endl;

			data = (void *)malloc(binarySize);

			f.seekg(binaryDataStart);
			f.read((char *)data, binarySize);

			//cout << "First element: " << *(int *)data << endl;

			break;
		}

		int pos = line.find(" ");

		metaData[line.substr(0, pos)] = boost::algorithm::trim_copy(line.substr(pos));

		binaryDataStart = f.tellg();
	}

	int nSamples = atoi(std::string(metaData["num_spikes"]).c_str());
	int nSamplesPerSpike = atoi(std::string(metaData["samples_per_spike"]).c_str());
	int nChannels = atoi(std::string(metaData["num_chans"]).c_str());

	//std::cout << "Number of spikes: " << nSamples << "." << std::endl;

	assert(nChannels == 4);
	assert(nSamplesPerSpike == 50);

	InputDataset* dataset = new InputDataset(nSamplesPerSpike*nChannels, nSamples, nChannels);

	int index = 0;
	for(int spike = 0; spike < nSamples; ++spike) {
		for(int channel = 0; channel < nChannels; ++channel) {
			index += 4; // skip timestamp

			for(int sample = 0; sample < nSamplesPerSpike; ++sample) {
				dataset->data[spike][channel*nSamplesPerSpike + sample] = (double)*((char *)data + (index + sample) );

			}
			index += nSamplesPerSpike;
		}

	}


	//DataExporter e = DataExporter();
	//e.saveSpikeData(dataset, "albert1.data", 1000);

	if(data != NULL)
		free(data);

	if (f.is_open())
		f.close();

	return dataset;
}

QStringList DataLoader::filetypes() {
	QStringList l;
	l << "Ascii" << "Axona";
	return l;
}


void DataLoader::loadCutFile(const char *filename, const std::vector<std::vector<double> > &points, std::vector<std::vector<int> > &lastComputedClusterIds, std::vector<std::vector<int> > &cutfileIds)
{
	std::ifstream f;
	std::string line;
	f.open(filename, std::ios::in);
	if(!f.is_open()) {
		std::cerr << "Unable to open file \"" << filename << "\"." << std::endl;
		return;
	}
	int numTimesDataLoaded = 0;
	std::vector<int> numSpikesInCluster(100); //FIXME: 100 is just a really big number so we don't have to count; clusters are 3-20 usually.
	int numClusters = -1;
	while (getline(f, line)) {
		if(line.find("n_clusters") != std::string::npos) {
			std::vector<std::string> tokens;
			boost::algorithm::split(tokens, line, boost::is_any_of(" "));
			std::istringstream is(tokens[1]);
			is >> numClusters;
			std::cout << "Number of clusters: " << numClusters << std::endl; // Always (?) 20
		}
		if(line.find("Exact_cut_for") != std::string::npos) {
			std::vector<std::string> tokens;
			boost::algorithm::split(tokens, line, boost::is_any_of(" "));
			std::istringstream is(tokens[3]);
			unsigned int numSpikes;
			is >> numSpikes;

			std::cout << "Number of spikes: " << numSpikes << std::endl;

			assert(++numTimesDataLoaded == 1);
			assert(numClusters != -1);

			cutfileIds.resize(numClusters);

			int index = 0;
			while (getline(f, line)) {
				boost::tokenizer<> tok(line);
				for(boost::tokenizer<>::iterator beg=tok.begin(); beg!=tok.end();++beg) {
					unsigned int id = atoi((*beg).c_str());
					cutfileIds[id].push_back(index);
					++index;

					if(id >= numSpikesInCluster.size())
						numSpikesInCluster.resize(numSpikesInCluster.size() + 50);

					numSpikesInCluster[id] += 1;
				}
			}
		}
	}
	for(std::vector<std::vector<int> >::iterator it = cutfileIds.begin(); it != cutfileIds.end(); ++it)
		if( (*it).empty()) {
			cutfileIds.erase(it);
			--it;
		}

	if(!lastComputedClusterIds.empty()) {
		//generate mean of both to compare

		int numDimensions = points[0].size();
		int numAutoCutClusters = lastComputedClusterIds.size();
		int numManualCutClusters = cutfileIds.size();
		int averageAutoWaveformsSize = numAutoCutClusters * numDimensions;
		int averageManualWaveformsSize = numManualCutClusters * numDimensions;

		double *autoCutMean = new double[averageAutoWaveformsSize];
		memset(autoCutMean, 0, sizeof(double) * averageAutoWaveformsSize);

		double *manualCutMean = new double[averageManualWaveformsSize];
		memset(manualCutMean, 0, sizeof(double) * averageManualWaveformsSize);

		//for(std::vector<std::vector<int> >::const_iterator clusterIt = lastComputedClusterIds.begin(); clusterIt != lastComputedClusterIds.end(); ++clusterIt) {
		//	for(std::vector<int>::const_iterator spikeIt = (*clusterIt).begin(); spikeIt != (*clusterIt).end(); ++spikeIt) {
		for(int cluster = 0; cluster < numAutoCutClusters; ++cluster)
			for(unsigned int point = 0; point < lastComputedClusterIds[cluster].size(); ++point)
				for(int dimension = 0; dimension < numDimensions; ++dimension)
					autoCutMean[cluster * numDimensions + dimension ] += points[lastComputedClusterIds[cluster][point]][dimension];

		for(int cluster = 0; cluster < numManualCutClusters; ++cluster)
			for(unsigned int point = 0; point < cutfileIds[cluster].size(); ++point)
				for(int dimension = 0; dimension < numDimensions; ++dimension)
					manualCutMean[cluster * numDimensions + dimension ] += points[ cutfileIds[cluster][point]][dimension];

		for(int cluster = 0; cluster < numAutoCutClusters; ++cluster)
			for(int dimension = 0; dimension < numDimensions; ++dimension )
				autoCutMean[cluster * numDimensions + dimension] /= lastComputedClusterIds[cluster].size();

		for(int cluster = 0; cluster < numManualCutClusters; ++cluster)
			for(int dimension = 0; dimension < numDimensions; ++dimension )
				manualCutMean[cluster * numDimensions + dimension] /= cutfileIds[cluster].size();


		std::vector<int> ordering(numAutoCutClusters);
		for(int i = 0; i < numAutoCutClusters; ++i)
			ordering[i] = i;

		similaritySumOfDifferences(manualCutMean, numManualCutClusters, autoCutMean, numAutoCutClusters, numDimensions, ordering);
/*
		for(int i = 0; i < ordering.size(); i++)
			std::cout << "Lol: " << ordering[i] << std::endl;

		std::vector<std::vector<int> > temp(lastComputedClusterIds.size());
		for(int i = 0; i < lastComputedClusterIds.size(); ++i)
				temp[i] = lastComputedClusterIds[i];

		for(int i = 0; i < numManualCutClusters; ++i) {
			if(i != ordering[i]) {
				std::cout << "Swapping " << i << " for " << ordering[i] << std::endl;
				//swap(temp[i], temp[ordering[i]]);
				temp[i].swap(temp[ordering[i]]);
			} else
				std::cout << "Keeping " << i << std::endl;

			temp[i] = lastComputedClusterIds[ordering[i]];
		}
		for(int i = numManualCutClusters; i < lastComputedClusterIds.size(); ++i) {
			std::cout << "Also keeping " << i << std::endl;
		}

		//for(int i = numManualCutClusters; i < lastComputedClusterIds.size(); ++i)
		//	temp.push_back(lastComputedClusterIds[i]);

		lastComputedClusterIds = temp;*/


		delete[] autoCutMean;
		delete[] manualCutMean;

	}
}
