#include "sdfilter.h"
#include <math.h>
#include <iostream>

SdFilter::SdFilter(InputDataset *inputData) {
	this->inputData = inputData;
	meanSpike.resize(inputData->nDimensions, 0);
	stdDev.resize(inputData->nDimensions, 0);
}

std::vector<int> *SdFilter::findNoiseSpikeIndexes(std::vector<int>  cluster, double stdDeviation) {
	std::vector<int> *nonNoiseIndexes = new std::vector<int>();

	for(unsigned int dimIndex = 0; dimIndex < inputData->nDimensions; dimIndex++) {
		meanSpike[dimIndex] = 0.0;
		stdDev[dimIndex] = 0.0;
	}


	// Calculate mean
	for(std::vector<int>::const_iterator it = cluster.begin(); it != cluster.end(); ++it) {
		int spikeIndex = *it;
		for(unsigned int dimIndex = 0; dimIndex < inputData->nDimensions; dimIndex++) {
			meanSpike[dimIndex] += inputData->data[spikeIndex][dimIndex];
		}
	}

	for(unsigned int dimIndex = 0; dimIndex < meanSpike.size(); dimIndex++) {
		meanSpike[dimIndex] /= cluster.size();
	}

	// Calculate SD
	for(std::vector<int>::const_iterator it = cluster.begin(); it != cluster.end(); ++it) {
		int spikeIndex = *it;
		for(unsigned int dimIndex = 0; dimIndex < inputData->nDimensions; dimIndex++) {
			double meanDiff = inputData->data[spikeIndex][dimIndex] - meanSpike[dimIndex];
			stdDev[dimIndex] += pow(meanDiff, 2);
		}
	}

	for(unsigned int dimIndex = 0; dimIndex < meanSpike.size(); dimIndex++) {
		stdDev[dimIndex] = sqrt(stdDev[dimIndex] / cluster.size());
	}

	// Check which are outside threshold
	for(std::vector<int>::const_iterator it = cluster.begin(); it != cluster.end(); ++it) {
		int spikeIndex = *it;
		bool shouldBeAdded = true;
		for(unsigned int dimIndex = 0; dimIndex < inputData->nDimensions; dimIndex++) {
			if(fabs(meanSpike[dimIndex] - inputData->data[spikeIndex][dimIndex]) > stdDeviation * stdDev[dimIndex] ) { // TODO how many Std devs
				//std::cout << "Noise: " << spikeIndex << std::endl;
				shouldBeAdded = false;
				break;
			}
		}
		if(shouldBeAdded)
			nonNoiseIndexes->push_back(spikeIndex);
	}

	return nonNoiseIndexes;
}
