#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QProgressDialog>
#include "data/inputdataset.h"
#include "spc/spc.h"
#include "spc/kmeans.h"
#include <qwt_plot_curve.h>
#include "susceptibilitythread.h"
#include <QLabel>

#include "datasetviewer.h"

namespace Ui {
    class MainWindow;
}

class MainWindow : public QMainWindow {
    Q_OBJECT
public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();

protected:
    void changeEvent(QEvent *e);

    InputDataset *inputData;
    InputDataset *reducedInputData;

	//TODO: ideally, these are just one polymorphed class. But requires some rewrite.. So hackedyhack.
	int currentClusteringIndex;
	enum currentClusteringIndexEnum { CLUSTER_NONE = -1, SPC, KMEANS };
	Spc *spc;
	KMeans *km;
	ClusteringAlgorithm *getCurrentClusteringAlgorithm();

	InputDataset *clusteredData;

private:
    Ui::MainWindow *ui;
	QProgressDialog *progressDialog;
	QTime *calculateStart;
	QTimer *progressTimer;

	SusceptibilityThread *susceptibilityThread;

	double lastFrac;
	double lastRemaining;

private slots:
	void on_export3dButton_clicked();
 void on_fmeasureButton_clicked();
 void on_lRatioButton_clicked();
 void on_silhouetteButton_clicked();
 void on_useReducedAsInput_clicked();
 void on_compareButton_clicked();
	void on_reductionChooser_performReduction(int index);
	void on_susceptibilityPlot_temperatureChanged(double newTemperature);
    void on_plotSusceptibilityButton_clicked();
    void on_actionAbout_triggered();
    void on_fileChooser_openFile(QString filename, int index);
	void susceptibilityThreadFinished();
	void updateProgress();

	DatasetViewer* viewDataset(InputDataset *dataset);
	void cancelSusceptibilityPlot();
	void on_plotSpikeClustersButton_clicked();
	void on_clusterButton_clicked();
	void on_clusteringTypeComboBox_currentIndexChanged(QString selected);
	void on_plotScatterButton_clicked();
	void on_saveCutFileButton_clicked();
	void on_loadCutFileBtn_clicked();
	void on_pretendCutFileButton_clicked();
};

#endif // MAINWINDOW_H
