#include "energypcareduction.h"

#include "pcareduction.h"
#include "scalebyenergyreduction.h"

#include <vector>
#include <math.h>
#include <iostream>

#include "macros.h"

EnergyPcaReduction::EnergyPcaReduction()
{

}

static double energyForSpike(const std::vector<double> *spike) {
	double squareSum = 0;
	for(std::vector<double>::const_iterator it = spike->begin(); it != spike->end(); it++) {
		squareSum += (*it) * (*it);
	}
	return sqrt(squareSum) / spike->size();
}

InputDataset* EnergyPcaReduction::reduce(const InputDataset *input) {
	InputDataset *outData = new InputDataset(2* input->nChannels, input->nSamples, 1);
	outData->nChannels = input->nChannels;

	PcaReduction pca;
	ScaleByEnergyReduction scaler;

	for(unsigned int channel=0;channel < input->nChannels; channel++) {
		InputDataset *channelData = input->extractChannel(channel);

		InputDataset *scaledData = scaler.reduce(channelData);
		//WHEREAMI();
		//		std::cout << "channel data: " << std::endl;
		//		channelData->print();
		InputDataset *pcaData = pca.reduce(scaledData, 1);
		delete scaledData;


		for(unsigned int spike=0;spike<input->nSamples; spike++) {
			outData->data[spike][2*channel] = energyForSpike(&(channelData->data[spike]));
			outData->data[spike][2*channel + 1] = pcaData->data[spike][0]; // First PCA component
		}
		delete channelData;

		delete pcaData;
	}

	return outData;
}

InputDataset* EnergyPcaReduction::reverse(const InputDataset *input) {
	return NULL;
}
