#include "pcaperchannelreduction.h"

#include "pcareduction.h"
#include "scalebyenergyreduction.h"

PcaPerChannelReduction::PcaPerChannelReduction()
{
}

InputDataset* PcaPerChannelReduction::reduce(const InputDataset *input, int numComponentsPerChannel) {
	if(numComponentsPerChannel == -1) {
		numComponentsPerChannel = 1; // Default
	}

	InputDataset *outData = new InputDataset(numComponentsPerChannel * input->nChannels, input->nSamples, input->nChannels);
	outData->nChannels = input->nChannels;

	PcaReduction pca;
	ScaleByEnergyReduction scaler;

	for(unsigned int channel=0;channel < input->nChannels; channel++) {
		InputDataset *channelData = input->extractChannel(channel);

		InputDataset *scaledData = scaler.reduce(channelData);
		//WHEREAMI();
		//std::cout << "channel data: " << std::endl;
		//channelData->print();
		InputDataset *pcaData = pca.reduce(scaledData, numComponentsPerChannel);
		delete scaledData;
		delete channelData;

		for(unsigned int spike=0;spike<input->nSamples; spike++) {
			for(int comp = 0; comp < numComponentsPerChannel; comp++) {
				outData->data[spike][numComponentsPerChannel*channel + comp] = pcaData->data[spike][comp];

			}
		}

		delete pcaData;
	}

	return outData;
}

InputDataset* PcaPerChannelReduction::reduce(const InputDataset *input) {
	return reduce(input, -1);
}

InputDataset* PcaPerChannelReduction::reverse(const InputDataset *input) {
	return NULL;
}
