#include "waveletreductionfirst.h"

#include <assert.h>
#include <string.h>
#include <iostream>
#include <math.h>

WaveletReductionFirst::WaveletReductionFirst()
{
	outDimensions = 16;

	waveletType = gsl_wavelet_daubechies;
	k = 8;

}

InputDataset *WaveletReductionFirst::reduce(const InputDataset *input) {
	/*if(outDimensions >= input->nDimensions) {
		std::cerr << "No reduction performed - already as few (or fewer) dimensions." << std::endl;
		return new InputDataset(*input);
	}*/


	int inputSampleSize = input->data[0].size();
	int n = 1<< (int)ceil(log2(inputSampleSize)); // pow2-padded

	InputDataset *outData = new InputDataset(outDimensions, input->nSamples, input->nChannels);
	outData->resizeVectorsToParameters();

	gsl_wavelet *wavelet = gsl_wavelet_alloc(waveletType, k); //TODO which to choose - supported ones are 4,6,8,...,20. See http://wavelets.pybytes.com/wavelet/db8/
	gsl_wavelet_workspace *work = gsl_wavelet_workspace_alloc(n);

	assert(wavelet != NULL);
	assert(work != NULL);

	double *data = new double[n];

	std::vector <std::vector <double > > transformedData;

	transformedData.resize( input->nSamples, std::vector<double>( n, 0) );

	// TODO could be parallelized with some effort
	for(unsigned int sampleIndex=0;sampleIndex<input->nSamples;sampleIndex++) {
		memset(data, 0, n*sizeof(double));
		std::copy(input->data[sampleIndex].begin(), input->data[sampleIndex].end(), data);

		gsl_wavelet_transform_forward(wavelet, data, 1, n, work);
		for(unsigned int i=0;i < outDimensions; i++) {
			outData->data[sampleIndex][i] = data[i];
		}
	}

	gsl_wavelet_free (wavelet);
	gsl_wavelet_workspace_free (work);
	delete[] data;

	outData->isReduced = true;
	outData->reduceClass = this;
	return outData;
}

InputDataset *WaveletReductionFirst::reverse(const InputDataset *input) {
	int n = 256; // pow2-padded
	int outN = 200;

	InputDataset *outData = new InputDataset(n, input->nSamples, input->nChannels);
	outData->resizeVectorsToParameters();

	gsl_wavelet *wavelet = gsl_wavelet_alloc(waveletType, k); // K/2 vanishing points
	gsl_wavelet_workspace *work = gsl_wavelet_workspace_alloc(n);

	assert(wavelet != NULL);
	assert(work != NULL);

	double *data = new double[n];

	for(unsigned int sampleIndex=0;sampleIndex<input->nSamples;sampleIndex++) {
		memset(data, 0, n*sizeof(double));
		std::copy(input->data[sampleIndex].begin(), input->data[sampleIndex].end(), data);

		gsl_wavelet_transform_inverse(wavelet, data, 1, n, work);
		for(int i=0;i < outN; i++) {
			outData->data[sampleIndex][i] = data[i];
		}
	}

	gsl_wavelet_free (wavelet);
	gsl_wavelet_workspace_free (work);
	delete[] data;
	outData->reduceClass = this;
	return outData;
}

WaveletReductionFirst::~WaveletReductionFirst() {

}
