#include "waveletreductionfirstperchannel.h"
#include <assert.h>
#include <string.h>
#include <iostream>
#include <math.h>

WaveletReductionFirstPerChannel::WaveletReductionFirstPerChannel()
{
	outDimensions = 16;
	waveletType = gsl_wavelet_daubechies;
	k = 8;
}

InputDataset *WaveletReductionFirstPerChannel::reduce(const InputDataset *input) {
	int channels = input->nChannels;
	int channelWidth = input->nDimensions/channels;

	int n = 1<< (int)ceil(log2(channelWidth)); // pow2-padded

	InputDataset *outData = new InputDataset(outDimensions, input->nSamples, input->nChannels);
	outData->resizeVectorsToParameters();

	gsl_wavelet *wavelet = gsl_wavelet_alloc(waveletType, k); //TODO which to choose - supported ones are 4,6,8,...,20. See http://wavelets.pybytes.com/wavelet/db8/
	gsl_wavelet_workspace *work = gsl_wavelet_workspace_alloc(n);

	assert(wavelet != NULL);
	assert(work != NULL);

	double *data = new double[n];

	for(unsigned int sampleIndex=0;sampleIndex<input->nSamples;sampleIndex++) {
		for(int channelIndex = 0; channelIndex < channels; channelIndex++) {
			memset(data, 0, n*sizeof(double));
			int startIndex = channelIndex * channelWidth;
			std::copy(input->data[sampleIndex].begin() + startIndex, input->data[sampleIndex].begin() + startIndex + channelWidth, data);

			gsl_wavelet_transform_forward(wavelet, data, 1, n, work);
			int outChannelWidth =  outDimensions/channels;
			for(int i=0;i < outChannelWidth; i++) {
				outData->data[sampleIndex][channelIndex*outChannelWidth + i] = data[i];
			}
		}
	}

	gsl_wavelet_free (wavelet);
	gsl_wavelet_workspace_free (work);
	delete[] data;

	outData->reduceClass = this;
	outData->isReduced = true;
	return outData;
}

InputDataset *WaveletReductionFirstPerChannel::reverse(const InputDataset *input) {
	int n = 64; // pow2-padded 50 samples
	int outN = 200;
	int channels = input->nChannels;
	int channelWidth = input->nDimensions / channels;

	InputDataset *outData = new InputDataset(outN, input->nSamples, input->nChannels);
	outData->resizeVectorsToParameters();

	gsl_wavelet *wavelet = gsl_wavelet_alloc(waveletType, k);
	gsl_wavelet_workspace *work = gsl_wavelet_workspace_alloc(n);

	assert(wavelet != NULL);
	assert(work != NULL);

	double *data = new double[n];

	for(unsigned int sampleIndex=0;sampleIndex<input->nSamples;sampleIndex++) {
		for(int channelIndex=0;channelIndex < channels; channelIndex++) {
			memset(data, 0, n*sizeof(double));

			int startIndex = channelIndex * channelWidth;
			std::copy(input->data[sampleIndex].begin() + startIndex, input->data[sampleIndex].begin() + startIndex + channelWidth, data);

			gsl_wavelet_transform_inverse(wavelet, data, 1, n, work);
			int outChannelWidth = 50;
			for(int i=0;i < outChannelWidth; i++) {
				outData->data[sampleIndex][channelIndex * outChannelWidth + i] = data[i];
			}
		}
	}

	gsl_wavelet_free (wavelet);
	gsl_wavelet_workspace_free (work);
	delete[] data;

	return outData;
}

WaveletReductionFirstPerChannel::~WaveletReductionFirstPerChannel() {

}
