#include "waveletreductionhighest.h"

#include "gsl/gsl_sort.h"
#include <string.h>
#include <math.h>
#include <assert.h>
#include <iostream>

WaveletReductionHighest::WaveletReductionHighest()
{
	outDimensions = 4;
	//TODO config parameters in constructor or other place?
	waveletType = gsl_wavelet_daubechies;
	k = 8;
}

InputDataset *WaveletReductionHighest::reduce(const InputDataset *input)
{
	if(outDimensions >= input->nDimensions) {
		std::cerr << "No reduction performed - already as few (or fewer) dimensions." << std::endl;
		return new InputDataset(*input);
	}

	int inputSampleSize = input->data[0].size();
	int n = 1<< (int)ceil(log2(inputSampleSize)); // pow2-padded

	InputDataset *outData = new InputDataset(outDimensions, input->nSamples, input->nChannels);
	outData->resizeVectorsToParameters();

	gsl_wavelet *wavelet = gsl_wavelet_alloc(waveletType, k);
	gsl_wavelet_workspace *work = gsl_wavelet_workspace_alloc(n);

	assert(wavelet != NULL);
	assert(work != NULL);

	size_t *p = new size_t[n];

	double *amplitudeSum = new double[n];
	memset(amplitudeSum, 0, n*sizeof(double));

	double *data = new double[n];


	std::vector <std::vector <double > > transformedData;

	transformedData.resize( input->nSamples, std::vector<double>( n, 0) );

	// TODO store all these values, and pick them from here
	for(unsigned int sampleIndex=0;sampleIndex<input->nSamples;sampleIndex++) {
		memset(data, 0, n*sizeof(double));
		std::copy(input->data[sampleIndex].begin(), input->data[sampleIndex].end(), data);

		gsl_wavelet_transform_forward(wavelet, data, 1, n, work);

		for (int i = 0; i < n; i++)
		{
			amplitudeSum[i] += fabs (data[i]);

			transformedData[sampleIndex][i] = data[i];
		}	
		//memcpy(&transformedData[sampleIndex], data, sizeof(double) * n);

	}


	// Find most important indices
	gsl_sort_index (p, amplitudeSum, 1, n);

	// Then transform and keep the values
	for(unsigned int sampleIndex=0;sampleIndex<input->nSamples;sampleIndex++) {
		for (unsigned int i = 0; i < outDimensions; i++)
		{
			outData->data[sampleIndex][i] = transformedData[sampleIndex][ p[i] ];
		}

	}

	gsl_wavelet_free (wavelet);
	gsl_wavelet_workspace_free (work);
	delete[] data;
	delete[] amplitudeSum;
	delete[] p;

	outData->isReduced = true;
	outData->reduceClass = this;
	return outData;
}

InputDataset* WaveletReductionHighest::reverse(const InputDataset *input) {
	//TODO this is not a reverse
	return new InputDataset(*input);
}

WaveletReductionHighest::~WaveletReductionHighest() {

}
