#include "clustercollection.h"
#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <string.h>
#include <vector>
#include <stack>
#include <algorithm>
#include <assert.h>


ClusterCollection::ClusterCollection(int nPoints) {
	this->nPoints = nPoints;
	clusterMemberships = new int[nPoints];
	reset();

}

ClusterCollection::~ClusterCollection() {
	delete[] clusterMemberships;
}

void ClusterCollection::reset() {
	clusters.clear();
	assert(clusters.size() == 0);
	assignedClusters = 0;

	for (unsigned int i = 0; i < nPoints; i++) {
		clusterMemberships[i] = -1;
	}
	mergeClusterLists.clear();
	mergeTargets.clear();
}

int ClusterCollection::startCluster() {
	assignedClusters++;
	clusters.resize(assignedClusters); //TODO: this resizes on every starC.. not good. BUT IT WORKS!
	return assignedClusters - 1;
}

void ClusterCollection::assign(int clusterIndex, int pointIndex) {
	clusterMemberships[pointIndex] = clusterIndex;

	//std::vector<int>::iterator it;
	//it = std::find(clusters[clusterIndex].begin(), clusters[clusterIndex].end(), pointIndex);
	//assert(it == clusters[clusterIndex].end());

	clusters[clusterIndex].push_back(pointIndex);
}

bool ClusterCollection::isAssignedToCluster(int index) {
	return clusterMemberships[index] != -1;
}

int ClusterCollection::getContainingCluster(int index) {
	return clusterMemberships[index];
}

int ClusterCollection::getAssignedClustersCount() {
	return assignedClusters;
}


void ClusterCollection::printClusters() {
	for (unsigned int i = 0; i < clusters.size(); i++) {
		std::cout << "Cluster " << i << ": ";

		for (std::vector<int>::const_iterator it = clusters[i].begin(); it != clusters[i].end(); it++) {
			std::cout << *it << " ";
		}
		std::cout << std::endl;
	}
}

void ClusterCollection::markMergeClustersForPointIndexes(int a, int b) {
	int clusterIndexA = getContainingCluster(a);
	int clusterIndexB = getContainingCluster(b);
	markMergeClusters(clusterIndexA, clusterIndexB);
}

void ClusterCollection::markMergeClusters(int clusterIndexA, int clusterIndexB) {
	bool clusterAHasTarget = mergeTargets.find(clusterIndexA) != mergeTargets.end();
	bool clusterBHasTarget = mergeTargets.find(clusterIndexB) != mergeTargets.end();

	if(clusterAHasTarget && clusterBHasTarget) {
		// move into A
		int targetIndex = mergeTargets[clusterIndexA];
		int sourceIndex = mergeTargets[clusterIndexB];
		if(targetIndex == sourceIndex)
			return; // Already merged
		for(std::vector<int>::const_iterator it = mergeClusterLists[sourceIndex].begin(); it < mergeClusterLists[sourceIndex].end(); it++) {
			mergeTargets[*it] = targetIndex;
		}
		mergeClusterLists[targetIndex].insert(mergeClusterLists[targetIndex].end(), mergeClusterLists[sourceIndex].begin(), mergeClusterLists[sourceIndex].end());
		mergeClusterLists[sourceIndex].clear();

	} else if(clusterAHasTarget) {
		mergeTargets[clusterIndexB] = mergeTargets[clusterIndexA];
		mergeClusterLists[mergeTargets[clusterIndexB]].push_back(clusterIndexB);
	} else if(clusterBHasTarget) {
		mergeTargets[clusterIndexA] = mergeTargets[clusterIndexB];
		mergeClusterLists[mergeTargets[clusterIndexA]].push_back(clusterIndexA);
	} else {
		int newMoveIndex = mergeClusterLists.size();
		mergeTargets[clusterIndexA] = mergeTargets[clusterIndexB] = newMoveIndex;
		mergeClusterLists.resize(newMoveIndex+1);

		mergeClusterLists[newMoveIndex].push_back(clusterIndexA);
		mergeClusterLists[newMoveIndex].push_back(clusterIndexB);
	}
}

void ClusterCollection::doMerge() {
	for(std::vector<std::vector<int> >::iterator it = mergeClusterLists.begin(); it != mergeClusterLists.end(); it++) {
		std::vector<int> *mergeList = &*it;
		if(mergeList->size() == 0) {
			continue; // No merging to be done for empty lists
		}
		int totalSize = 0;
		int largestSize = 0;
		int targetClusterIndex = 0;
		for(std::vector<int>::iterator itt = mergeList->begin(); itt != mergeList->end(); itt++) {
			int currentSize = clusters[*itt].size();
			if(currentSize > largestSize) {
				largestSize = currentSize;
				targetClusterIndex = *itt;
			}
			totalSize += currentSize;
		}
		clusters[targetClusterIndex].reserve(totalSize);
		for(std::vector<int>::iterator itt = mergeList->begin(); itt != mergeList->end(); itt++) {
			int currentIndex = *itt;
			if(currentIndex != targetClusterIndex) {
				clusters[targetClusterIndex].insert(clusters[targetClusterIndex].end(), clusters[currentIndex].begin(), clusters[currentIndex].end());
				clusters[currentIndex].clear();
			}
		}
	}
}

void ClusterCollection::removeEmptyClusters() {
	std::stack<int> emptyClusters;
	for(unsigned int i=0;i<clusters.size();i++) {
		if(clusters[i].empty()) {
			emptyClusters.push(i);
		}
	}

	while(!emptyClusters.empty()) {
		int eraseIndex = emptyClusters.top();
		emptyClusters.pop();
		clusters.erase(clusters.begin() + eraseIndex);
	}
}
