#ifndef KMEANS_H
#define KMEANS_H

#include <tbb/tbb.h>

#include <vector>
#include "clusteringalgorithm.h"

#define Uint32 unsigned int

struct Centroid {
	//float pos[2];
	//pos will be num channels * num samples wide
	//don't actually initialize this - point it to one huge array instead
	float *pos;
	float color[3];
};

class KMeans : public ClusteringAlgorithm
{
public:
	KMeans(std::vector<std::vector<double> > points, int numCentroids, int cutoff);
	~KMeans();

	void initCentroids();
	void classifyPoints(int start, int stop, int *pointsUpdated, float *tempNewClusters, unsigned int *tempNewClusterSize);
	void OnLoop();

	void convertToVectorFormat();


	int convergenceStatus;
	//FIXME: don't do this here
	const static float CONVERGENCE_DELTA = 0.000001f;

	Centroid *centroids;
	float *rawCentroids;

	Uint32 numPoints;
	int numCentroids;
	int cutoff;

	int numIterationsRun;

	int loopSerial();
	int loopParallel();
};

class KMeansTask : public tbb::task {
public:

	KMeansTask(KMeans *_km, int start, int _numPoints, int *_numUpdatedPoints, int _serialThreshold, float *_tempNewClusters, Uint32 *_tempNewClusterSize);

	tbb::task *execute();

	int serialThreshold;
	int numPoints;
	int *numUpdatedPoints;
	int start;
	KMeans *km;

	float *tempNewClusters;
	Uint32 *tempNewClusterSize;
};

#endif // KMEANS_H
