#ifndef SPC_H_
#define SPC_H_

#include <ANN/ANN.h>

#include <vector>

#include "xyplot.h"
#include "xycollectionplot.h"

#include "clusteringalgorithm.h"

#include "randomgenerator.h"

#include "tbb/tbb.h"


class Spc : public ClusteringAlgorithm {
public:
	Spc(std::vector<std::vector<double> > points);
	~Spc();

	XyPlot* plotSusceptibility(double minTemperature, double maxTemperature, double step);
	double calculateSusceptibility(const double temperature, bool determineSpinSpinCorrelation, RandomGenerator *innerRandom, bool printClusteringResults = false, int* spins = NULL);

	double finishness;
	XyCollectionPlot *correlationClusterSizesPlot;

	XyPlot *susceptibilityPlot;
	void cancelSusceptibilityPlot();

	RandomGenerator *randomGen;
	bool shouldCancelSusceptibilityPlot;

	double initialGuessTemperature;

private:
	// Parameters for SPC
	static const int nMonteCarloIterations = 2000; // N
	static const int nSpinStates = 20; // q
	static const double theta = 0.5;
	static const int nWarmupIterations = 100;

	static const double annErrorToleranceFraction = 0.0;

	double thetaForGij;


	void buildKdtree();
	void determineClosestNeighborsAndInteractionStrengths(unsigned int nNearestNeighbors);
	void printPoints();

	ANNpointArray kdPoints;
	ANNkd_tree *kdTree;

	std::vector<std::vector<int> > neighborIndexes;
	std::vector<std::vector<double> > neighborDistances;
	std::vector<std::vector<double> > interactionStrengths; //J_ij

	void removeImmutualNeighborsAndBackNeighbors();
	void searchKdTreeAndCopyNeighborData(unsigned int nNearestNeighbors);
	void calculateInteractionStrengths(unsigned int pairs, double avgDistance);
	std::vector<std::vector<double> > calculateProbabilityValues(const double & temperature);

	void checkConnectedNeighborGraph();
	void addMstToNeighborGraph();
	void connectPoints(int fromIndex, int toIndex);
	double distance(int fromIndex, int toIndex);

	double calculateEnergy(int *spins);



};

class PlotSusceptibility {
	public:
	PlotSusceptibility(Spc *spc, int *iterationsPerformed, double minTemperature, double step, int numResults);
	void operator()(tbb::blocked_range<size_t> &r) const;

private:
	Spc *spc;
	int *iterationsPerformed;
	double minTemperature;
	double step;
	int numResults;

};

#endif /* SPC_H_ */
