#include "xyseries.h"
#include <limits>

XYSeries::XYSeries(QVector<QPointF> points) :
	QwtPointSeriesData()
{
	this->points = points;

	xMin = std::numeric_limits<double>::max();
	xMax = std::numeric_limits<double>::min();
	yMin = std::numeric_limits<double>::max();
	yMax = std::numeric_limits<double>::min();
	for(int i=0;i<points.size();i++) {
		double x = points[i].x();
		double y = points[i].y();
		if(x>xMax)
			xMax = x;
		if(x<xMin)
			xMin = x;
		if(y<yMin)
			yMin = y;
		if(y>yMax)
			yMax = y;
	}

}

QPointF XYSeries::sample(size_t i) const {
	return points[i];
}

const QVector<QPointF> XYSeries::samples() const {
	return points;
}

QRectF XYSeries::boundingRect() const {
	return QRectF( QPointF(xMin, yMin), QPointF(xMax, yMax));
}

size_t XYSeries::size() const {
	return points.size();
}
